@@ -57,7 +57,6 @@
 
 #include <linux/module.h>
 #include <linux/moduleparam.h>
-#include <linux/init.h>
 #include <linux/crc32.h>
 #include <linux/usb.h>
 #include <linux/firmware.h>
@@ -307,6 +306,34 @@ enum {
 #define FW_GET_BYTE(p) (*((__u8 *) (p)))
 
 #define FW_DIR "/*(DEBLOBBED)*/"
+#define EAGLE_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define ADI930_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define EAGLE_I_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define EAGLE_II_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define EAGLE_III_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define EAGLE_IV_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+
+#define DSP4I_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define DSP4P_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define DSP9I_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define DSP9P_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define DSPEI_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define DSPEP_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define FPGA930_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+
+#define CMV4P_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMV4PV2_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMV4I_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMV4IV2_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMV9P_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMV9PV2_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMV9I_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMV9IV2_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMVEP_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMVEPV2_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMVEI_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+#define CMVEIV2_FIRMWARE FW_DIR "/*(DEBLOBBED)*/"
+
 #define UEA_FW_NAME_MAX 30
 #define NB_MODEM 4
 
@@ -694,26 +721,26 @@ err:
 static int uea_load_firmware(struct usb_device *usb, unsigned int ver)
 {
 	int ret;
-	char *fw_name = FW_DIR "/*(DEBLOBBED)*/";
+	char *fw_name = EAGLE_FIRMWARE;
 
 	uea_enters(usb);
 	uea_info(usb, "pre-firmware device, uploading firmware\n");
 
 	switch (ver) {
 	case ADI930:
-		fw_name = FW_DIR "/*(DEBLOBBED)*/";
+		fw_name = ADI930_FIRMWARE;
 		break;
 	case EAGLE_I:
-		fw_name = FW_DIR "/*(DEBLOBBED)*/";
+		fw_name = EAGLE_I_FIRMWARE;
 		break;
 	case EAGLE_II:
-		fw_name = FW_DIR "/*(DEBLOBBED)*/";
+		fw_name = EAGLE_II_FIRMWARE;
 		break;
 	case EAGLE_III:
-		fw_name = FW_DIR "/*(DEBLOBBED)*/";
+		fw_name = EAGLE_III_FIRMWARE;
 		break;
 	case EAGLE_IV:
-		fw_name = FW_DIR "/*(DEBLOBBED)*/";
+		fw_name = EAGLE_IV_FIRMWARE;
 		break;
 	}
 
@@ -869,19 +896,19 @@ static int request_dsp(struct uea_softc *sc)
 
 	if (UEA_CHIP_VERSION(sc) == EAGLE_IV) {
 		if (IS_ISDN(sc))
-			dsp_name = FW_DIR "/*(DEBLOBBED)*/";
+			dsp_name = DSP4I_FIRMWARE;
 		else
-			dsp_name = FW_DIR "/*(DEBLOBBED)*/";
+			dsp_name = DSP4P_FIRMWARE;
 	} else if (UEA_CHIP_VERSION(sc) == ADI930) {
 		if (IS_ISDN(sc))
-			dsp_name = FW_DIR "/*(DEBLOBBED)*/";
+			dsp_name = DSP9I_FIRMWARE;
 		else
-			dsp_name = FW_DIR "/*(DEBLOBBED)*/";
+			dsp_name = DSP9P_FIRMWARE;
 	} else {
 		if (IS_ISDN(sc))
-			dsp_name = FW_DIR "/*(DEBLOBBED)*/";
+			dsp_name = DSPEI_FIRMWARE;
 		else
-			dsp_name = FW_DIR "/*(DEBLOBBED)*/";
+			dsp_name = DSPEP_FIRMWARE;
 	}
 
 	ret = reject_firmware(&sc->dsp_firm, dsp_name, &sc->usb_dev->dev);
@@ -1357,10 +1384,8 @@ static int uea_stat_e1(struct uea_softc *sc)
 		/* release the dsp firmware as it is not needed until
 		 * the next failure
 		 */
-		if (sc->dsp_firm) {
-			release_firmware(sc->dsp_firm);
-			sc->dsp_firm = NULL;
-		}
+		release_firmware(sc->dsp_firm);
+		sc->dsp_firm = NULL;
 	}
 
 	/* always update it as atm layer could not be init when we switch to
@@ -1496,10 +1521,8 @@ static int uea_stat_e4(struct uea_softc *sc)
 		/* release the dsp firmware as it is not needed until
 		 * the next failure
 		 */
-		if (sc->dsp_firm) {
-			release_firmware(sc->dsp_firm);
-			sc->dsp_firm = NULL;
-		}
+		release_firmware(sc->dsp_firm);
+		sc->dsp_firm = NULL;
 	}
 
 	/* always update it as atm layer could not be init when we switch to
@@ -1929,7 +1952,7 @@ static int load_XILINX_firmware(struct uea_softc *sc)
 	int ret, size, u, ln;
 	const u8 *pfw;
 	u8 value;
-	char *fw_name = FW_DIR "/*(DEBLOBBED)*/";
+	char *fw_name = FPGA930_FIRMWARE;
 
 	uea_enters(INS_TO_USBDEV(sc));
 
@@ -2238,10 +2261,9 @@ static void uea_stop(struct uea_softc *sc)
 	usb_free_urb(sc->urb_int);
 
 	/* flush the work item, when no one can schedule it */
-	flush_work_sync(&sc->task);
+	flush_work(&sc->task);
 
-	if (sc->dsp_firm)
-		release_firmware(sc->dsp_firm);
+	release_firmware(sc->dsp_firm);
 	uea_leaves(INS_TO_USBDEV(sc));
 }
 
@@ -2758,3 +2780,4 @@ module_usb_driver(uea_driver);
 MODULE_AUTHOR("Damien Bergamini/Matthieu Castet/Stanislaw W. Gruszka");
 MODULE_DESCRIPTION("ADI 930/Eagle USB ADSL Modem driver");
 MODULE_LICENSE("Dual BSD/GPL");
+/*(DEBLOBBED)*/
