@@ -47,8 +47,6 @@ int request_firmware_nowait(
 	void (*cont)(const struct firmware *fw, void *context));
 
 void release_firmware(const struct firmware *fw);
-int cache_firmware(const char *name);
-int uncache_firmware(const char *name);
 #else
 static inline int request_firmware(const struct firmware **fw,
 				   const char *name,
@@ -68,15 +66,13 @@ static inline void release_firmware(const struct firmware *fw)
 {
 }
 
-static inline int cache_firmware(const char *name)
-{
-	return -ENOENT;
-}
+#endif
 
-static inline int uncache_firmware(const char *name)
-{
-	return -EINVAL;
-}
+#ifdef CONFIG_FW_LOADER_USER_HELPER
+int request_firmware_direct(const struct firmware **fw, const char *name,
+			    struct device *device);
+#else
+#define request_firmware_direct	request_firmware
 #endif
 
 #ifndef _LINUX_LIBRE_FIRMWARE_H
@@ -114,6 +110,18 @@ maybe_reject_firmware(const struct firmware **fw,
 	else
 		return request_firmware(fw, name, device);
 }
+static inline int
+reject_firmware_direct(const struct firmware **fw,
+		const char *name, struct device *device)
+{
+	const struct firmware *xfw = NULL;
+	int retval;
+	report_missing_free_firmware(dev_name(device), NULL);
+	retval = request_firmware_direct(&xfw, NONFREE_FIRMWARE, device);
+	if (!retval)
+		release_firmware(xfw);
+	return -EINVAL;
+}
 static inline void
 discard_rejected_firmware(const struct firmware *fw, void *context)
 {
